import numpy as np
from read_cifar import *
from matplotlib import pyplot as plt

def distance_matrix(mat1, mat2):
    
    norm1 = np.sum(mat1**2, axis=1, keepdims=True)
    norm2 = np.sum(mat2**2, axis=1, keepdims=True)
    dot_products = np.dot(mat1, mat2.T)
    dists = np.sqrt(norm1 - 2 * dot_products + norm2.T)

    return dists


def knn_predict(dists, labels_train, k):
    
    
    predicted_labels = np.zeros(dists.shape[0], dtype=int)
    
    for i in range(0,dists.shape[0],1):
        nearest_indices = np.argsort(dists[i])[:k]
        nearest_labels=[labels_train[i] for i in nearest_indices]    
        predicted_class=max(nearest_labels,key=nearest_labels.count)                  
        predicted_labels[i]=predicted_class  
    
    return predicted_labels

def evaluate_knn(data_train, labels_train, data_test, labels_test, k):
   
    dists=distance_matrix(data_train,data_test)
    predicted_labels=knn_predict(dists,labels_train,k)
    accuracy = (np.sum(predicted_labels == labels_test)) / len(labels_test)

    return accuracy



def acc_graph():
  
  axis=[]
  result=[]
  data_path = r"C:\Intel\Desktop\DeepLearning\image-classification\data\cifar-10-batches-py"
  allData,allLabels = read_cifar(data_path)
  data_train,labels_train, data_test,labels_test = split_dataset(allData,allLabels, split=0.9) 
  
  for i in range(1,20,1):
    axis.append(i)
    acc=evaluate_knn(data_train,labels_train, data_test,labels_test,i)             
    result.append(acc)
  plt.plot(axis,result)
  plt.title("the variation of the accuracy as a function of k")
  plt.xlabel("Number of neighbors")
  plt.ylabel("Accuracy")
  plt.show()
  plt.savefig(r"C:\Intel\Desktop\DeepLearning\image-classification\results")
    
if __name__ == "__main__":
  acc_graph()
