from read_cifar import *
from matplotlib import pyplot as plt


def segmoid(x):
  return  1/(1 + np.exp(-x))

def deriv_segmoid(x):
  return segmoid(x)*(1-segmoid(x))

def softmax(x):
  e_x = np.exp(x) 
  return e_x / e_x.sum(axis=1, keepdims=True)

def learn_once_mse(w1,b1,w2,b2,data,targets,learning_rate):
    
    a0=data
    a1=segmoid(np.matmul(a0, w1) + b1)
    a2=segmoid(np.matmul(a1,w2) + b2) #The predicted classes

    #Calculate the different partial derivatives
    dc_da2=2*(a2-targets)
    dc_dz2=np.multiply(np.multiply(a2,(1-a2)),dc_da2)
    dc_dw2=np.matmul(a1.T,dc_dz2)
    dc_db2=dc_dz2

    dc_da1=np.matmul(dc_dz2,w2.T)
    dc_dz1=np.multiply(np.multiply(a1,(1-a1)),dc_da1)
    dc_dw1=np.matmul(a0.T,dc_dz1)
    dc_db1=dc_dz1

    #application of the backpropagation of the gradient
    w1=w1-learning_rate*dc_dw1
    w2=w2-learning_rate*dc_dw2
    b1=b1-learning_rate*dc_db1
    b2=b2-learning_rate*dc_db2

    # Forward pass
    z1 = np.matmul(a0, w1) + b1   
    a1 = segmoid(z1)              
    z2 = np.matmul(a1, w2) + b2   
    a2 = segmoid(z2)              
    predictions = a2              

    # Compute loss (MSE)
    loss = np.mean(np.square(predictions - targets))

    return (w1,b1,w2,b2,loss)

def one_hot(NDarray):
  
  result = np.zeros((NDarray.shape[0],int(np.max(NDarray)+1)))
  for i in range(0,NDarray.shape[0],1):
    result[i,int(NDarray[i])]=1
  return result


def cross_entropy(classes,prob):
  loss=-np.sum(np.multiply(classes,np.log(prob)))
  return loss/float(prob.shape[0])


def learn_once_cross_entropy(w1,b1,w2,b2,data,labels_train,learning_rate):
  
  a0=data                            
  labels_train=one_hot(labels_train)           
  a1=segmoid(np.matmul(a0, w1) + b1) 
  a2=softmax(np.matmul(a1,w2) + b2)  
  nb_rows=data.shape[0]

  #gradient descent optimization
  
  dc_dz2=(a2-labels_train)/data.shape[0]
  dc_dw2=np.matmul(a1.T,dc_dz2)
  dc_db2=np.dot(np.ones(nb_rows),dc_dz2) 

  dc_da1=np.matmul(dc_dz2,w2.T)
  dc_dz1=np.multiply(np.multiply(a1,(1-a1)),dc_da1)
  dc_dw1=np.matmul(a0.T,dc_dz1)
  dc_db1= np.dot(np.ones(nb_rows),dc_dz1) 

  #application of the backpropagation of the gradient
  w1=w1-learning_rate*dc_dw1
  w2=w2-learning_rate*dc_dw2
  b1=b1-learning_rate*dc_db1
  b2=b2-learning_rate*dc_db2


  # Forward pass
  z1 = np.matmul(a0, w1) + b1       
  a1 = segmoid(z1)                  
  z2 = np.matmul(a1, w2) + b2       
  a2 = softmax(z2)                  
  # Compute loss (Cross entropy loss)
  loss = cross_entropy(labels_train,a2)
  return (w1,b1,w2,b2,loss)

def train_mlp(w1,b1,w2,b2,data_train,labels_train,learning_rate,num_epoch):

  train_accuracies=[]   
  for i in range(0,num_epoch,1):
    (w1,b1,w2,b2,loss)=learn_once_cross_entropy(w1,b1,w2,b2,data_train,labels_train,learning_rate)
    
    a0=data_train                              
    z1 = np.matmul(a0, w1) + b1                
    a1 = segmoid(z1)                           
    z2 = np.matmul(a1, w2) + b2                
    a2 = softmax(z2)                           
    predict = np.argmax(a2,axis=1)         

    train_accuracies.append((np.sum(predict == labels_train)/predict.shape[0])*100)

  return (w1,w2,b1,b2,train_accuracies)

def test_mlp(w1,b1,w2,b2,data_test,labels_test):
  
  a0=data_test
  z1 = np.matmul(a0, w1) + b1  
  a1 = segmoid(z1)  
  z2 = np.matmul(a1, w2) + b2  
  a2 = softmax(z2) 
  predict = np.argmax(a2,axis=1)  
  return (np.sum(predict == labels_test)/predict.shape[0])*100

def run_mlp_training(data_train, labels_train, data_test, labels_test,d_h,learning_rate ,num_epoch ):

  d_in = data_train.shape[1]              
  d_out = 10                               
  w1 = 2 * np.random.rand(d_in, d_h) - 1  
  b1 = np.zeros((1, d_h))                 
  w2 = 2 * np.random.rand(d_h, d_out) - 1 
  b2 = np.zeros((1, d_out))               


  (w1,w2,b1,b2,train_accuracies)=train_mlp(w1,b1,w2,b2,data_train,labels_train,learning_rate,num_epoch)
  
  test_acc=test_mlp(w1,b1,w2,b2,data_test,labels_test)

  return (train_accuracies,test_acc)


if __name__ == "__main__":
      
  dir_batches =  r'C:\Intel\Desktop\DeepLearning\image-classification\data\cifar-10-batches-py'
  (data,labels)=read_cifar(dir_batches)
  (data_train, labels_train, data_test, labels_test)=split_dataset(data,labels,0.9)
  
  d_in = data_train.shape[1]              
  d_out = 10  

  w1 = 2 * np.random.rand(d_in, 64) - 1  
  b1 = np.zeros((1, 64))  
  w2 = 2 * np.random.rand(64, d_out) - 1  
  b2 = np.zeros((1, d_out))  
  acc=train_mlp(w1,b1,w2,b2,data_train,labels_train,0.1,100)[4]
  epochs =[i+1 for i in range(0,100,1)]
  
  plt.plot(epochs,acc)
  plt.title("the evolution of learning accuracy across learning epochs")
  plt.xlabel("epochs")
  plt.ylabel("Accuracy")
  plt.show()
  plt.savefig(r"C:\Intel\Desktop\DeepLearning\image-classification\results")