import os
import numpy as np
import pickle


def unpickle(path):
 
  with open(path, 'rb') as fo:
    dict = pickle.load(fo, encoding='bytes')
  return dict

def read_cifar_batch(path):

    batch_data = unpickle(path)
    data = batch_data[b'data']
    labels = batch_data[b'labels']
    data = np.array(data, dtype=np.float32)
    labels = np.array(labels, dtype=np.int64)

    return data, labels

def read_cifar(path):
    file_list = os.listdir(path)
    unwanted_files = ['data_batch_1', 'batches.meta', 'readme.html']
    #initilize the Matrix data and labels with the data of the first batch
    (Matrix_Data, allLabels) = read_cifar_batch(os.path.join(path, 'data_batch_1'))
    file_list.remove('data_batch_1')
    for i in file_list:
        if i not in unwanted_files:
            (data, labels) = read_cifar_batch(os.path.join(path, i))
            Matrix_Data = np.concatenate((Matrix_Data, data), axis=0)
            allLabels = np.concatenate((allLabels, labels), axis=0)

    return (Matrix_Data, allLabels)


def split_dataset(data, labels, split):
    if split < 0 or split > 1:
        raise ValueError("The 'split' factor is a float between 0 and 1")

    # Determine the split indices
    split_idx = int(len(data) * split)

    # Shuffle the data and labels using the same random order
    shuffled_indices = np.random.permutation(len(data))
    data_shuffled = data[shuffled_indices]
    labels_shuffled = labels[shuffled_indices]

    # Split the data and labels into training and test sets
    data_train = data_shuffled[:split_idx]
    labels_train = labels_shuffled[:split_idx]
    data_test = data_shuffled[split_idx:]
    labels_test = labels_shuffled[split_idx:]

    return data_train, labels_train, data_test, labels_test




if __name__ == "__main__":
    
   batch_path = r"C:\Intel\Desktop\DeepLearning\image-classification\data\cifar-10-batches-py\data_batch_1"
   data_path = r"C:\Intel\Desktop\DeepLearning\image-classification\data\cifar-10-batches-py"
   data, labels = read_cifar_batch(batch_path)
   allData,allLabels = read_cifar(data_path)
   print("Data shape:", data.shape)
   print("Labels shape:", labels.shape)
   data_train,labels_train, data_test,labels_test = split_dataset(allData,allLabels, split=0.9)
   print("Data_train:", data_train)
   print("Labels_train:", labels_train)
   print("Data_test:", data_test)
   print("Labels_test:", labels_test)